@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* ss.h:
-**  Copyright (C) 2015-2017 Mednafen Team
+**  Copyright (C) 2015-2019 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -26,6 +26,8 @@
 
 #include <trio/trio.h>
 
+using namespace Mednafen;
+
 namespace MDFN_IEN_SS
 {
  enum
@@ -63,12 +65,24 @@ namespace MDFN_IEN_SS
   SS_DBG_SCSP 	   = (1U << 28),
   SS_DBG_SCSP_REGW = (1U << 29),
  };
-#ifdef MDFN_SS_DEV_BUILD
- extern uint32 ss_dbg_mask;
+#ifdef MDFN_ENABLE_DEV_BUILD
+ MDFN_HIDE extern uint32 ss_dbg_mask;
 #else
  enum { ss_dbg_mask = 0 };
 #endif
 
+#if 1
+ enum
+ {
+  HORRIBLEHACK_NOSH2DMALINE106	 = (1U << 0),
+  HORRIBLEHACK_NOSH2DMAPENALTY   = (1U << 1),
+  HORRIBLEHACK_VDP1VRAM5000FIX	 = (1U << 2),
+  HORRIBLEHACK_VDP1RWDRAWSLOWDOWN= (1U << 3),
+  HORRIBLEHACK_VDP1INSTANT	 = (1U << 4),
+ };
+ MDFN_HIDE extern uint32 ss_horrible_hacks;
+#endif
+
  static INLINE void SS_DBG_Dummy(const char* format, ...) { }
  #define SS_DBG(which, ...) ((MDFN_UNLIKELY(ss_dbg_mask & (which))) ? (void)trio_printf(__VA_ARGS__) : SS_DBG_Dummy(__VA_ARGS__))
  #define SS_DBGTI(which, ...) ((MDFN_UNLIKELY(ss_dbg_mask & (which))) ? ((void)trio_printf(__VA_ARGS__), (void)trio_printf(" @Line=0x%03x, HPos=0x%03x, memts=%d\n", VDP2::PeekLine(), VDP2::PeekHPos(), SH7095_mem_timestamp)) : SS_DBG_Dummy(__VA_ARGS__))
@@ -92,9 +106,9 @@ namespace MDFN_IEN_SS
 
  class SH7095;
 
- extern SH7095 CPU[2];	// for smpc.cpp
+ MDFN_HIDE extern SH7095 CPU[2];	// for smpc.cpp
 
- extern int32 SH7095_mem_timestamp;
+ MDFN_HIDE extern int32 SH7095_mem_timestamp;
 
  void SS_RequestMLExit(void);
  void ForceEventUpdates(const sscpu_timestamp_t timestamp);
@@ -136,7 +150,7 @@ namespace MDFN_IEN_SS
   ss_event_handler event_handler;
  };
 
- extern event_list_entry events[SS_EVENT__COUNT];
+ MDFN_HIDE extern event_list_entry events[SS_EVENT__COUNT];
 
  #define SS_EVENT_DISABLED_TS			0x40000000
  void SS_SetEventNT(event_list_entry* e, const sscpu_timestamp_t next_timestamp);
